/**
*	gcc -c quantizacao.c -I /usr/include/opencv -lcv -lm -lhighgui -lcvaux
**/
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <cv.h>
#include <highgui.h>
#include <ctype.h>

void normalize_hist(long int *hist, double *histnorm, int N) {
    int i;
    long int sum = 0;
    double e = 0.001;
    for (i = 0; i < N ; i++) {
	  sum+= hist[i]*hist[i];
    }
    for (i = 0; i < N ; i++) {
	  histnorm[i] = sqrt(hist[i]/((double)sum+e));
    }
}

void quantization(IplImage *img, IplImage *img_quant, int numQuant)
{
	int i,j;
	int height   = img->height;       // altura da imagem (eixo x)
	int width    = img->width;   	  // largura da imagem (eixo y)  
	int step     = img->widthStep;    // tamanho em bytes de uma linha da matriz da imagem
	unsigned char *data = (uchar *)img->imageData;
	int step2 = img_quant->widthStep; 
	unsigned char *data2 = (uchar *)img_quant->imageData; 
	
	//Encontrando os 2 bits mais significativos 8b -> 2 MSB
	for (i = 0; i < height ; i++)	{
	    for (j = 0; j < width; j++) 	    {
	      // operador XXXXXXXX & 11000000 equivalente a XXXXXXXX AND 11000000
	      // operador 01000000 >> 2 equivalente a deslocamento de 2 bits para a direita = 00010000 
	      // rr000000 R - deslocar os bits em 2 posicoes - 00rr0000
	      // gg000000 G - deslocar os bits em 4 posicoes - 0000gg00
	      // bb000000 B - deslocar os bits em 6 posicoes - 000000bb
	      unsigned char C1 = (data[i*step+j*3+0] & 192)>>2;// 2 BMS posicao 00xx0000
	      unsigned char C2 = (data[i*step+j*3+1] & 192)>>4;// 2 BMS posicao 0000xx00
	      unsigned char C3 = (data[i*step+j*3+2] & 192)>>6;// 2 BMS posicao 000000xx
	      data2[i*step2+j*1] = C1 | C2 | C3; // operador | equivalente a 'OU', faz a fusao dos tres componentes
	    }     
	}
}

void histogram(IplImage *img_quant, double *histogram_norm, int numQuant, int height, int width)
{
	int i, j;
	long int *hist = calloc(numQuant,sizeof(long int));
	uchar *data = (uchar *) img_quant->imageData;
	int step = img_quant->widthStep;
	for (i = 0; i < height; i++) 
	{
	  for (j = 0; j < width; j++) 
	  {
	      hist[data[i*step+j]]++;  // incrementa o valor da cor encontrada no histograma
	  }     
	}
	normalize_hist(hist, histogram_norm, numQuant);
}


void GCH(IplImage *img, double *descriptor, int numQuant)
{
   int i,j;
   int height   = img->height;       // altura da imagem (eixo x)
   int width    = img->width;        // largura da imagem (eixo y)  
   int step     = img->widthStep;    // tamanho em bytes de uma linha da matriz da imagem
   uchar *data = (uchar*) img->imageData;

   IplImage * img_quant = cvCreateImage(cvGetSize(img), IPL_DEPTH_8U, 1);
   
   long int * c_border = calloc(numQuant, sizeof(long int));
   long int * c_interior = calloc(numQuant, sizeof(long int));

   quantization(img, img_quant, numQuant);
   histogram(img_quant, descriptor, numQuant, height, width);
}

void BIC(IplImage *img, double *descriptor, int numQuant, int normalize)
{
   int i,j;
   IplImage * img_quant = cvCreateImage(cvGetSize(img), IPL_DEPTH_8U, 1);
   quantization(img, img_quant, numQuant);
   int height   = img_quant->height;       // altura da imagem (eixo x)
   int width    = img_quant->width;        // largura da imagem (eixo y)  
   int step     = img_quant->widthStep;    // tamanho em bytes de uma linha da matriz da imagem
   uchar *data = (uchar*) img_quant->imageData, aux;

   // counting arrays
   long int * c_border = calloc(numQuant, sizeof(long int));
   long int * c_interior = calloc(numQuant, sizeof(long int));
   
   for (i = 0; i < height; i++) {
	for (j = 0; j < width; j++) {
		unsigned char aux = data[i*step+j];
		if (i > 0 && j > 0 && j < width && i < height) 
		{
			if ((data[i*step+(j-1)]-aux)==0 && 
			    (data[i*step+(j+1)]-aux)==0 && 
			    (data[(i+1)*step+j]-aux)==0 && 
			    (data[(i-1)*step+j]-aux)==0) 
			{
				c_interior[aux]++;
		      	}
			else
				c_border[aux]++;
		}
		else
			c_border[aux]++;
	}     
   }

   // real-valued arrays
   double * border   = calloc(numQuant,sizeof(double));
   double * interior = calloc(numQuant,sizeof(double));
   if (normalize == 1) {
      normalize_hist(c_interior, interior, numQuant);
      normalize_hist(c_border, border, numQuant);
      for (i = 0; i < numQuant; i++) {
	    descriptor[i] = border[i];
	    descriptor[i+numQuant] = interior[i];
      }
   }
   else {
      for (i = 0; i < numQuant; i++) {
	    descriptor[i] = (double)c_border[i];
	    descriptor[i+numQuant] = (double)c_interior[i];
      }
   }
   free(c_border);
   free(c_interior);
   free(border);
   free(interior);
}


double dManhattan(double *q, double *p, int tam)
{
	int i;
	double somat=0.0;
	for(i=0;i<tam;i++) {
		somat = somat + fabs(q[i]-p[i]);
	}
	somat = somat/(double)tam;
	return somat;
}

double dEuclid(double *q, double *p, int tam)
{
	int i;
	double somat=0;
	for(i=0;i<tam;i++) {
		somat = somat + pow((q[i]-p[i]),2);
	}
	somat = sqrt(somat);
	return somat;
}

double fAuxLog(double x) {
	if (x == 0)
	    return 0;
	else 
	    if (x < 1)
		return 1;
	    else {
		return (int)(log(x)/log(2))+1;
	    }
}

double dLog(double *q, double *p, int tam, int factor)
{
	int i;
	double somat=0;
	for(i=0;i<tam;i++) {
		somat = somat + abs(fAuxLog(q[i]*factor)-fAuxLog(p[i]*factor));
	}
	somat = sqrt(somat);
	return somat;
}

